@@ -25,7 +25,7 @@
  */
 
 /*
- *  Copyright (C) 2004-2013 Max-Planck-Society
+ *  Copyright (C) 2004-2015 Max-Planck-Society
  *  Author: Martin Reinecke
  */
 
@@ -59,9 +59,21 @@ Candidates for testing the validity of the Healpix routines:
 #include "geom_utils.h"
 #include "walltimer.h"
 #include "announce.h"
+#include "compress_utils.h"
+#include "moc.h"
+#include "moc_fitsio.h"
 
 using namespace std;
 
+#define UNITTESTS
+
+#ifdef UNITTESTS
+int errcount=0;
+#define FAIL(a) {a; if (++errcount>100) planck_fail("unit test errors");}
+#else
+#define FAIL(a) a;
+#endif
+
 const int nsamples = 1000000;
 
 planck_rng rng;
@@ -82,6 +94,362 @@ void random_zphi (double &z, double &phi)
 template<typename I> string bname()
   { return string("(basetype: ")+type2typename<I>()+")"; }
 
+
+template<typename I> rangeset<I> randomRangeSet(int num, I start, int dist)
+  {
+  rangeset<I> rs;
+  I curval=start;
+  for (int i=0; i<num; ++i)
+    {
+    I v1=curval+1+(rng.int_rand_uni()%dist);
+    I v2=v1+1+(rng.int_rand_uni()%dist);
+    rs.append(v1,v2);
+    curval=v2;
+    }
+  return rs;
+  }
+template<typename I> Moc<I> randomMoc(int num, I start, int dist)
+  {
+  Moc<I> moc;
+  I curval=start+(I(1)<<(2*Moc<I>::maxorder));
+  for (int i=0; i<num; ++i)
+    {
+    I v1=curval+1+(rng.int_rand_uni()%dist);
+    I v2=v1+1+(rng.int_rand_uni()%dist);
+    moc.addPixelRange(Moc<I>::maxorder,v1,v2);
+    curval=v2;
+    }
+  return moc;
+  }
+
+template<typename I> rangeset<I> makeRS(const string &input)
+  {
+  istringstream is(input);
+  rangeset<I> res;
+  I a,b;
+  while (is)
+    {
+    is>>a>>b;
+    planck_assert (is||is.eof(),"aargh");
+    if (is) res.append(a,b);
+    }
+  return res;
+  }
+template<typename I> void rsOps(const rangeset<I> &a, const rangeset<I> &b)
+  {
+  planck_assert(!b.overlaps(a.op_andnot(b)),"error");
+  planck_assert(a.op_or(b).nval()>=a.nval(),"error");
+  planck_assert(a.op_or(b).nval()>=b.nval(),"error");
+  planck_assert(a.op_and(b).nval()<=a.nval(),"error");
+  planck_assert(a.op_and(b).nval()<=b.nval(),"error");
+  planck_assert(a.op_or(b).contains(a),"error");
+  planck_assert(a.op_or(b).contains(b),"error");
+  planck_assert(!a.op_andnot(b).overlaps(b),"error");
+  planck_assert(!b.op_andnot(a).overlaps(a),"error");
+  planck_assert(a.op_or(b).nval()==a.nval()+b.nval()-a.op_and(b).nval(),
+    "error");
+  planck_assert(a.op_or(b).op_andnot(a.op_and(b)).nval()==
+                a.op_or(b).nval()-a.op_and(b).nval(),"error");
+  planck_assert(a.op_xor(b)==a.op_andnot(b).op_or(b.op_andnot(a)),"error");
+  }
+template<typename I> void check_rangeset()
+  {
+  cout << "testing rangeset " << bname<I>() << endl;
+  {
+  rangeset<I> b;
+  b.append(1,11);
+  planck_assert(b==makeRS<I>("1 11"),"mismatch");
+  b.append(10,15);
+  planck_assert(b==makeRS<I>("1 15"),"mismatch");
+  b.append(1,15);
+  planck_assert(b==makeRS<I>("1 15"),"mismatch");
+  b.append(7,15);
+  planck_assert(b==makeRS<I>("1 15"),"mismatch");
+  b.append(30,41);
+#if 0
+  planck_assert(b==makeRS<I>("1 15 30 41"),"mismatch");
+  try
+    {
+    b.append(29,31);
+    planck_fail("Should have raised an IllegalArgumentException");
+    }
+  catch (PlanckError E) {}
+#endif
+  }
+  {
+  rangeset<I> b=makeRS<I>("1 11 30 41");
+  planck_assert(!b.contains(0),"error");
+  planck_assert(b.contains(1),"error");
+  planck_assert(b.contains(5),"error");
+  planck_assert(b.contains(10),"error");
+  planck_assert(!b.contains(11),"error");
+  planck_assert(!b.contains(29),"error");
+  planck_assert(b.contains(30),"error");
+  planck_assert(b.contains(35),"error");
+  planck_assert(b.contains(40),"error");
+  planck_assert(!b.contains(41),"errror");
+  }
+  {
+  rangeset<I> b;
+  b.add(5, 11);
+  planck_assert(b==makeRS<I>("5 11"),"error");
+  b.add(1, 7);
+  planck_assert(b==makeRS<I>("1 11"),"error");
+  b.add(1, 11);
+  planck_assert(b==makeRS<I>("1 11"),"error");
+  b.add(30, 41);
+  planck_assert(b==makeRS<I>("1 11 30 41"),"error");
+  b.add(1, 11);
+  planck_assert(b==makeRS<I>("1 11 30 41"),"error");
+  b.add(-1,0);
+  planck_assert(b==makeRS<I>("-1 0 1 11 30 41"),"error");
+  b.add(-2,-1);
+  planck_assert(b==makeRS<I>("-2 0 1 11 30 41"),"error");
+  b.add(-2,-1);
+  planck_assert(b==makeRS<I>("-2 0 1 11 30 41"),"error");
+  b.add(2, 11);
+  planck_assert(b==makeRS<I>("-2 0 1 11 30 41"),"error");
+  b.add(1, 10);
+  planck_assert(b==makeRS<I>("-2 0 1 11 30 41"),"error");
+  b.add(15, 21);
+  planck_assert(b==makeRS<I>("-2 0 1 11 15 21 30 41"),"error");
+  }
+  {
+  rangeset<I> b = makeRS<I>("0 11 20 31");
+  b.remove(5,25);
+  planck_assert(b==makeRS<I>("0 5 25 31"),"error");
+  b.remove(31,32);
+  planck_assert(b==makeRS<I>("0 5 25 31"),"error");
+  b.remove(35,38);
+  planck_assert(b==makeRS<I>("0 5 25 31"),"error");
+  b.remove(-90,-80);
+  planck_assert(b==makeRS<I>("0 5 25 31"),"error");
+  b.remove(27,29);
+  planck_assert(b==makeRS<I>("0 5 25 27 29 31"),"error");
+  b.remove(25,26);
+  planck_assert(b==makeRS<I>("0 5 26 27 29 31"),"error");
+  b.remove(4,6);
+  planck_assert(b==makeRS<I>("0 4 26 27 29 31"),"error");
+  b.remove(-20,40);
+  planck_assert(b==makeRS<I>(""),"error");
+  b.remove(-20,40);
+  planck_assert(b==makeRS<I>(""),"error");
+  }
+  {
+  rangeset<I> b = makeRS<I>("0 11 20 31");
+  b.intersect(2,29);
+  planck_assert(b==makeRS<I>("2 11 20 29"),"error");
+  b.intersect(-8,50);
+  planck_assert(b==makeRS<I>("2 11 20 29"),"error");
+  b.intersect(2,50);
+  planck_assert(b==makeRS<I>("2 11 20 29"),"error");
+  b.intersect(2,29);
+  planck_assert(b==makeRS<I>("2 11 20 29"),"error");
+  b.intersect(-18,29);
+  planck_assert(b==makeRS<I>("2 11 20 29"),"error");
+  b.intersect(3,11);
+  planck_assert(b==makeRS<I>("3 11"),"error");
+  b = makeRS<I>("0 11 20 31");
+  b.intersect(3,15);
+  planck_assert(b==makeRS<I>("3 11"),"error");
+  b = makeRS<I>("0 11 20 31");
+  b.intersect(17,30);
+  planck_assert(b==makeRS<I>("20 30"),"error");
+  b = makeRS<I>("0 11 20 31");
+  b.intersect(11,20);
+  planck_assert(b==makeRS<I>(""),"error");
+  b = makeRS<I>("0 11 20 31");
+  b.intersect(-8,-7);
+  planck_assert(b==makeRS<I>(""),"error");
+  b = makeRS<I>("0 11 20 31");
+  b.intersect(31,35);
+  planck_assert(b==makeRS<I>(""),"error");
+  }
+  {
+  planck_assert(makeRS<I>("1 11 20 31 40 56")==
+                makeRS<I>("20 31 40 51").op_or
+                (makeRS<I>("1 11 45 56")),"error");
+  planck_assert(makeRS<I>("1 11 45 56")==
+                makeRS<I>("").op_or
+                (makeRS<I>("1 11 45 56")),"error");
+  planck_assert(makeRS<I>("1 11 45 56")==
+                makeRS<I>("1 11 45 56").op_or
+                (makeRS<I>("")),"error");
+  }
+  {
+  planck_assert(makeRS<I>("22 24 45 51")==
+                makeRS<I>("20 31 40 51").op_and
+                (makeRS<I>("1 11 22 24 45 56")),"error");
+  planck_assert(makeRS<I>("20 31 40 51 90 101 110 121 200 201")==
+                makeRS<I>("10 101 110 121 200 221").op_and
+                (makeRS<I>("20 31 40 51 90 201")),"error");
+  planck_assert(makeRS<I>("")==
+                makeRS<I>("20 31 40 51").op_and
+                (makeRS<I>("")),"error");
+  planck_assert(makeRS<I>("")==
+                makeRS<I>("").op_and
+                (makeRS<I>("20 31 40 51")),"error");
+  }
+  {
+  planck_assert(makeRS<I>("20 31 40 45")==
+                makeRS<I>("20 31 40 51").op_andnot
+                (makeRS<I>("1 11 45 56")),"error");
+  planck_assert(makeRS<I>("")==
+                makeRS<I>("").op_andnot
+                (makeRS<I>("1 11 45 56")),"error");
+  planck_assert(makeRS<I>("1 11 45 56")==
+                makeRS<I>("1 11 45 56").op_andnot
+                (makeRS<I>("")),"error");
+  }
+  {
+  rangeset<I> b = makeRS<I>("20 31 40 51");
+
+  planck_assert(!b.contains(0,11),"error");
+  planck_assert(!b.contains(10,21),"error");
+  planck_assert(!b.contains(19,20),"error");
+  planck_assert(b.contains(20,21),"error");
+  planck_assert(b.contains(21,22),"error");
+  planck_assert(b.contains(20,31),"error");
+  planck_assert(!b.contains(25,36),"error");
+  planck_assert(b.contains(30,31),"error");
+  planck_assert(!b.contains(31,32),"error");
+  planck_assert(!b.contains(35,38),"error");
+  planck_assert(!b.contains(35,46),"error");
+  planck_assert(b.contains(40,41),"error");
+  planck_assert(!b.contains(45,56),"error");
+  planck_assert(!b.contains(60,71),"error");
+  }
+  {
+  rangeset<I> b = makeRS<I>("20 31 40 51");
+
+  planck_assert(b.contains(makeRS<I>("20 31 40 51")),"error");
+  planck_assert(b.contains(makeRS<I>("20 21")),"error");
+  planck_assert(b.contains(makeRS<I>("50 51")),"error");
+  planck_assert(!b.contains(makeRS<I>("19 31 40 51")),"error");
+  planck_assert(!b.contains(makeRS<I>("20 31 40 52")),"error");
+  planck_assert(!b.contains(makeRS<I>("20 51")),"error");
+  planck_assert(!b.contains(makeRS<I>("0 1")),"error");
+  planck_assert(!b.contains(makeRS<I>("0 20 31 40 51 100")),"error");
+  }
+  {
+  rangeset<I> b = makeRS<I>("20 31 40 51");
+
+  planck_assert(!b.overlaps(0,11),"error");
+  planck_assert(b.overlaps(10,21),"error");
+  planck_assert(!b.overlaps(19,20),"error");
+  planck_assert(b.overlaps(20,21),"error");
+  planck_assert(b.overlaps(21,22),"error");
+  planck_assert(b.overlaps(20,31),"error");
+  planck_assert(b.overlaps(25,36),"error");
+  planck_assert(b.overlaps(30,37),"error");
+  planck_assert(!b.overlaps(31,32),"error");
+  planck_assert(!b.overlaps(35,38),"error");
+  planck_assert(b.overlaps(35,46),"error");
+  planck_assert(b.overlaps(40,41),"error");
+  planck_assert(b.overlaps(45,56),"error");
+  planck_assert(!b.overlaps(60,71),"error");
+  }
+  {
+  rangeset<I> b = makeRS<I>("20 31 40 51");
+
+  planck_assert(b.overlaps(makeRS<I>("20 31 40 51")),"error");
+  planck_assert(b.overlaps(makeRS<I>("20 21")),"error");
+  planck_assert(b.overlaps(makeRS<I>("50 51")),"error");
+  planck_assert(b.overlaps(makeRS<I>("19 31 40 51")),"error");
+  planck_assert(b.overlaps(makeRS<I>("20 31 40 52")),"error");
+  planck_assert(b.overlaps(makeRS<I>("20 51")),"error");
+  planck_assert(!b.overlaps(makeRS<I>("0 1")),"error");
+  planck_assert(!b.overlaps(makeRS<I>("0 20 31 40 51 100")),"error");
+  }
+  {
+  const int niter = 1000;
+  for (int iter=0; iter<niter; ++iter)
+    {
+    rangeset<I> a = randomRangeSet<I>(1000, 0, 100);
+    rangeset<I> b = randomRangeSet<I>(1000, 0, 100);
+    rangeset<I> c = randomRangeSet<I>(10, 10000, 100);
+    rsOps(a,b);
+    rsOps(a,c);
+    rsOps(c,a);
+    }
+  }
+  }
+template<typename I> void check_Moc()
+  {
+  cout << "testing MOC " << bname<I>() << endl;
+  Moc<I> moc;
+  moc.addPixelRange(0,4,5);
+  moc.addPixelRange(0,6,7);
+  moc.addPixelRange(2,4,17);
+  moc.addPixelRange(10,3000000,3000001);
+
+  planck_assert(moc==moc.complement().complement(),"error");
+  planck_assert(moc==Moc<I>::fromUniq(moc.toUniq()),"error");
+  planck_assert(moc.maxOrder()==10,"error");
+  Moc<I> xtmp = moc.degradedToOrder(8,false);
+  planck_assert(moc.contains(xtmp),"error");
+  planck_assert(!xtmp.contains(moc),"error");
+  planck_assert(xtmp.overlaps(moc),"error");
+  xtmp=moc.degradedToOrder(8,true);
+  planck_assert(!moc.contains(xtmp),"error");
+  planck_assert(xtmp.contains(moc),"error");
+  planck_assert(xtmp.overlaps(moc),"error");
+  planck_assert(moc==Moc<I>::fromCompressed(moc.toCompressed()),"error");
+#if 0
+  assertEquals("inconsistency",moc,MocUtil.mocFromString(" 0/4, 6 2/ \t 4 -16 10/3000000 \t\n "));
+  assertEquals("inconsistency",moc,MocUtil.mocFromString("0/6 2/ 5 2/4 2/6- 16 0/4  10/3000000"));
+  assertEquals("inconsistency",moc,MocUtil.mocFromString
+    ("{\"0\":[6] , \"2\": [5 ], \"2\":[  4,6,7,8,9,10,11,12,13,14,15,16], \"0\":[4],  \"10\":[3000000]}"));
+  assertEquals("inconsistency",moc,MocUtil.mocFromString(MocUtil.mocToStringASCII(moc)));
+  assertEquals("inconsistency",moc,MocUtil.mocFromString(MocUtil.mocToStringJSON(moc)));
+  ByteArrayOutputStream out= new ByteArrayOutputStream();
+  MocUtil.mocToFits(moc,out);
+  ByteArrayInputStream inp = new ByteArrayInputStream(out.toByteArray());
+  assertEquals("inconsistency",moc,MocUtil.mocFromFits(inp));
+#endif
+  {
+  tsize niter = 100;
+  Moc<I> full; full.addPixelRange(0,0,12);
+  Moc<I> empty;
+  for (tsize iter=0; iter<niter; ++iter)
+    {
+    Moc<I> a = randomMoc<I>(1000, 0, 100);
+    planck_assert(a.complement().complement()==a,"error");
+    planck_assert(!a.overlaps(a.complement()),"error");
+    planck_assert(a.op_or(a.complement())==full,"error");
+    planck_assert(a.op_and(a.complement())==empty,"error");
+#if 0
+    write_Moc_to_fits("!healpixtestmoctmp",a);
+    Moc<I> b;
+    read_Moc_from_fits("healpixtestmoctmp",b);
+    planck_assert(a==b,"FITS problem");
+#endif
+    }
+  }
+  }
+template<typename I> void check_compress()
+  {
+  cout << "testing interpolation coding " << bname<I>() << endl;
+  planck_assert(trailingZeros(4)==2,"error");
+  planck_assert(trailingZeros(5)==0,"error");
+  planck_assert(trailingZeros(int64(1)<<48)==48,"error");
+  for (tsize x=0; x<100; ++x)
+    {
+    rangeset<I> a = randomRangeSet<I>(1000, 0, 100);
+    rangeset<I> b;
+    for (tsize i=0; i<a.nranges(); ++i)
+      b.append(a.ivbegin(i)<<6,a.ivend(i)<<6);
+    const vector<I> &v=b.data();
+    obitstream obs;
+    interpol_encode(v.begin(),v.end(),obs);
+    vector<uint8> comp=obs.state();
+    vector<I> v2;
+    ibitstream ibs(comp);
+    interpol_decode(v2,ibs);
+    planck_assert(v==v2,"data mismatch");
+    }
+  }
+
 template<typename I> void check_ringnestring()
   {
   cout << "testing ring2nest(nest2ring(m))==m " << bname<I>() << endl;
@@ -92,7 +460,7 @@ template<typename I> void check_ringnestring()
       {
       I pix = I(rng.rand_uni()*base.Npix());
       if (base.ring2nest(base.nest2ring(pix))!=pix)
-        cout << "  PROBLEM: order = " << order << ", pixel = " << pix << endl;
+        FAIL(cout<<"  PROBLEM: order = "<<order<<", pixel = "<<pix<<endl)
       }
     }
   }
@@ -107,7 +475,7 @@ template<typename I> void check_nestpeanonest()
       {
       I pix = I(rng.rand_uni()*base.Npix());
       if (base.peano2nest(base.nest2peano(pix))!=pix)
-        cout << "  PROBLEM: order = " << order << ", pixel = " << pix << endl;
+        FAIL(cout<<"  PROBLEM: order = "<<order<<", pixel = "<<pix<<endl)
       }
     }
   }
@@ -125,10 +493,10 @@ template<typename I> void check_pixzphipix()
       I pix = I(rng.rand_uni()*base1.Npix());
       base1.pix2zphi(pix,z,phi);
       if (base1.zphi2pix(z,phi)!=pix)
-        cout << "  PROBLEM: order = " << order << ", pixel = " << pix << endl;
+        FAIL(cout<<"  PROBLEM: order = "<<order<<", pixel = "<<pix<<endl)
       base2.pix2zphi(pix,z,phi);
       if (base2.zphi2pix(z,phi)!=pix)
-        cout << "  PROBLEM: order = " << order << ", pixel = " << pix << endl;
+        FAIL(cout<<"  PROBLEM: order = "<<order<<", pixel = "<<pix<<endl)
       }
     }
   for (I nside=3; nside<(I(1)<<omax); nside+=nside/2+1)
@@ -140,7 +508,7 @@ template<typename I> void check_pixzphipix()
       I pix = I(rng.rand_uni()*base.Npix());
       base.pix2zphi(pix,z,phi);
       if (base.zphi2pix(z,phi)!=pix)
-        cout << "  PROBLEM: nside = " << nside << ", pixel = " << pix << endl;
+        FAIL(cout<<"  PROBLEM: nside = "<<nside<<", pixel = "<<pix<<endl)
       }
     }
   }
@@ -159,12 +527,12 @@ template<typename I> void check_zphipixzphi()
       random_zphi (z,phi);
       base1.pix2zphi(base1.zphi2pix(z,phi),z2,phi2);
       if (cosdist_zphi(z,phi,z2,phi2)<mincos)
-        cout << "  PROBLEM: order = " << order
-             << ", zphi = " << z << ", " << phi << endl;
+        FAIL(cout << "  PROBLEM: order = " << order
+                  << ", zphi = " << z << ", " << phi << endl)
       base2.pix2zphi(base2.zphi2pix(z,phi),z2,phi2);
       if (cosdist_zphi(z,phi,z2,phi2)<mincos)
-        cout << "  PROBLEM: order = " << order
-             << ", zphi = " << z << ", " << phi << endl;
+        FAIL(cout << "  PROBLEM: order = " << order
+                  << ", zphi = " << z << ", " << phi << endl)
       }
     }
   for (int nside=3; nside<(I(1)<<omax); nside+=nside/2+1)
@@ -177,8 +545,8 @@ template<typename I> void check_zphipixzphi()
       random_zphi (z,phi);
       base.pix2zphi(base.zphi2pix(z,phi),z2,phi2);
       if (cosdist_zphi(z,phi,z2,phi2)<mincos)
-        cout << "  PROBLEM: nside = " << nside
-             << ", zphi = " << z << ", " << phi << endl;
+        FAIL(cout << "  PROBLEM: nside = " << nside
+                  << ", zphi = " << z << ", " << phi << endl)
       }
     }
   }
@@ -194,9 +562,9 @@ template<typename I> void check_pixangpix()
       {
       I pix = I(rng.rand_uni()*base1.Npix());
       if (base1.ang2pix(base1.pix2ang(pix))!=pix)
-        cout << "  PROBLEM: order = " << order << ", pixel = " << pix << endl;
+        FAIL(cout<<"  PROBLEM: order = "<<order<<", pixel = "<<pix<<endl)
       if (base2.ang2pix(base2.pix2ang(pix))!=pix)
-        cout << "  PROBLEM: order = " << order << ", pixel = " << pix << endl;
+        FAIL(cout<<"  PROBLEM: order = "<<order<<", pixel = "<<pix<<endl)
       }
     }
   for (I nside=3; nside<(I(1)<<omax); nside+=nside/2+1)
@@ -206,7 +574,7 @@ template<typename I> void check_pixangpix()
       {
       I pix = I(rng.rand_uni()*base.Npix());
       if (base.ang2pix(base.pix2ang(pix))!=pix)
-        cout << "  PROBLEM: nside = " << nside << ", pixel = " << pix << endl;
+        FAIL(cout<<"  PROBLEM: nside = "<<nside<<", pixel = "<<pix<<endl)
       }
     }
   }
@@ -240,9 +608,9 @@ template<typename I> void check_neighbors()
         else
           {
           if (v_angle(base.pix2vec(nb[n]),pixpt)>maxang)
-            cout << " PROBLEM: order = " << order << ", pix = " << pix << endl;
+            FAIL(cout<<"  PROBLEM: order = "<<order<<", pix = "<<pix<<endl)
           if ((n>0) && (nb[n]==nb[n-1]))
-            cout << " PROBLEM: order = " << order << ", pix = " << pix << endl;
+            FAIL(cout<<"  PROBLEM: order = "<<order<<", pix = "<<pix<<endl)
           }
         }
       planck_assert((check<=1)||((order==0)&&(check<=2)),"too few neighbors");
@@ -260,7 +628,7 @@ template<typename I> void check_neighbors()
       base.neighbors(pix,nb);
       for (int n=0; n<8; ++n)
         if ((nb[n]>=0) && (v_angle(base.pix2vec(nb[n]),pixpt)>maxang))
-          cout << "  PROBLEM: nside = " << nside << ", pix = " << pix << endl;
+          FAIL(cout<<"  PROBLEM: nside = "<<nside<<", pix = "<<pix<<endl)
       }
     }
   }
@@ -277,7 +645,33 @@ void check_swap_scheme()
     map.swap_scheme();
     for (int m=0; m<map.Npix(); ++m)
       if (map[m]!=(m&0xFF))
-        cout << "  PROBLEM: order = " << order << ", pix = " << m << endl;
+        FAIL(cout<<"  PROBLEM: order = "<<order<<", pix = "<<m<<endl)
+    }
+  }
+
+void check_issue_229 (Healpix_Ordering_Scheme scheme)
+  {
+  cout << "checking issue #229" << endl;
+  int order=8;
+  Healpix_Map<bool> map (order,scheme);
+  map.fill(false);
+  Healpix_Map<vec3> vmap(order,scheme);
+  for (int m=0; m<vmap.Npix(); ++m)
+    vmap[m]=vmap.pix2vec(m);
+  rangeset<int> pixset;
+  pointing ptg(halfpi-0.1,0);
+  double rad=0.1;
+  map.query_disc(ptg,rad,pixset);
+  vec3 vptg=ptg;
+  double cosrad=cos(rad);
+  for (tsize j=0; j<pixset.nranges(); ++j)
+    for (int i=pixset.ivbegin(j); i<pixset.ivend(j); ++i)
+      map[i] = true;
+  for (int i=0; i<map.Npix(); ++i)
+    {
+    bool inside = dotprod(vmap[i],vptg)>cosrad;
+    if (inside^map[i])
+      FAIL(cout << "  PROBLEM: issue 229" << endl)
     }
   }
 
@@ -286,6 +680,7 @@ void check_query_disc_strict (Healpix_Ordering_Scheme scheme)
   cout << "testing whether all pixels found by query_disc() really" << endl
        << "lie inside the disk (and vice versa)" << endl;
   cout << "Ordering scheme: " << (scheme==RING ? "RING" : "NEST") << endl;
+  rng.seed(42);
   for (int order=0; order<=5; ++order)
     {
     Healpix_Map<bool> map (order,scheme);
@@ -302,16 +697,16 @@ void check_query_disc_strict (Healpix_Ordering_Scheme scheme)
       map.query_disc(ptg,rad,pixset);
       vec3 vptg=ptg;
       double cosrad=cos(rad);
-      for (tsize j=0; j<pixset.size(); ++j)
+      for (tsize j=0; j<pixset.nranges(); ++j)
         for (int i=pixset.ivbegin(j); i<pixset.ivend(j); ++i)
           map[i] = true;
       for (int i=0; i<map.Npix(); ++i)
         {
         bool inside = dotprod(vmap[i],vptg)>cosrad;
         if (inside^map[i])
-          cout << "  PROBLEM: order = " << order << ", ptg = " << ptg << endl;
+          FAIL(cout<<"  PROBLEM: order = "<<order<<", ptg = "<<ptg<<endl)
         }
-      for (tsize j=0; j<pixset.size(); ++j)
+      for (tsize j=0; j<pixset.nranges(); ++j)
         for (int i=pixset.ivbegin(j); i<pixset.ivend(j); ++i)
           map[i] = false;
       }
@@ -321,56 +716,50 @@ void check_query_disc_strict (Healpix_Ordering_Scheme scheme)
 template<typename I>void check_query_disc()
   {
   cout << "checking query_disc() " << bname<I>() << endl;
-  int omax=min(20,T_Healpix_Base<I>::order_max);
+  rng.seed(48);
+  int omax=min<int>(20,T_Healpix_Base<I>::order_max);
   for (int order=0; order<=omax; ++order)
     {
     T_Healpix_Base<I> rbase (order,RING), nbase (order,NEST);
-    rangeset<I> pr,pn;
+    rangeset<I> pixset;
     int niter=max(1,min(1000,100000>>order));
     for (int m=0; m<niter; ++m)
       {
       pointing ptg;
       random_dir (ptg);
       double rad = pi/1 * rng.rand_uni();
-      rbase.query_disc(ptg,rad,pr);
-      nbase.query_disc(ptg,rad,pn);
-      if (pn.nval()!=pr.nval())
-        cout << "PROBLEM: set size mismatch" << endl;
-      if (order<5) // takes too long for high resolutions
-        for (tsize i=0; i<pr.size(); ++i)
-          for (tsize j=pr.ivbegin(i);j<pr.ivend(i); ++j)
-            if (!pn.contains(rbase.ring2nest(j)))
-              cout << "PROBLEM: set mismatch" << endl;
-
+      rbase.query_disc(ptg,rad,pixset);
+      rangeset<I> pslast=pixset;
+      for (tsize fct=5; fct>0; --fct)
+        {
+        rangeset<I> psi;
+        rbase.query_disc_inclusive(ptg,rad,psi,fct);
+        if (!psi.contains(pslast))
+          cout << "  Potential problem: RING pixel sets inconsistent" << endl;
+        swap(pslast,psi);
+        }
+      I nval = pixset.nval();
+      nbase.query_disc(ptg,rad,pixset);
+      pslast=pixset;
       for (tsize fct=8; fct>0; fct>>=1)
         {
-        rangeset<I> pri,pni;
-        rbase.query_disc_inclusive(ptg,rad,pri,fct);
-        nbase.query_disc_inclusive(ptg,rad,pni,fct);
-        if (pni.nval()>pri.nval())
-          cout << "PROBLEM: RING inclusive < NEST inclusive" << endl;
-        if (pni.nval()<pri.nval())
-          cout << "Warning: RING inclusive > NEST inclusive "
-               << order << " " << fct << endl;
-        if (order<5) // takes too long for high resolutions
-          {
-          for (tsize i=0; i<pni.size(); ++i)
-            for (tsize j=pni.ivbegin(i);j<pni.ivend(i); ++j)
-              if (!pri.contains(rbase.nest2ring(j)))
-                cout << "PROBLEM: NEST inclusive not in RING inclusive" << endl;
-          for (tsize i=0; i<pn.size(); ++i)
-            for (tsize j=pn.ivbegin(i);j<pn.ivend(i); ++j)
-              if (!pni.contains(j))
-                cout << "PROBLEM: Inclusive not superset of normal" << endl;
-          }
+        rangeset<I> psi;
+        nbase.query_disc_inclusive(ptg,rad,psi,fct);
+        if (!psi.contains(pslast))
+          FAIL(cout << "  PROBLEM: NEST pixel sets inconsistent" << endl)
+        swap(pslast,psi);
         }
+      if (nval!=pixset.nval())
+        FAIL(cout << "  PROBLEM: number of pixels different: "
+                  << nval << " vs. " << pixset.nval() << endl)
       }
     }
   }
 template<typename I>void check_query_polygon()
   {
   cout << "checking query_polygon() " << bname<I>() << endl;
-  int omax=min(20,T_Healpix_Base<I>::order_max);
+  rng.seed(42);
+  int omax=min<int>(20,T_Healpix_Base<I>::order_max);
   for (int order=0; order<=omax; ++order)
     {
     T_Healpix_Base<I> rbase (order,RING), nbase (order,NEST);
@@ -384,18 +773,18 @@ template<typename I>void check_query_polygon()
       I nval = pixset.nval();
       nbase.query_polygon(corner,pixset);
       if (nval!=pixset.nval())
-        cout << "  PROBLEM: number of pixels different: "
-             << nval << " vs. " << pixset.nval() << endl;
+        FAIL(cout << "  PROBLEM: number of pixels different: "
+                  << nval << " vs. " << pixset.nval() << endl)
       rbase.query_polygon_inclusive(corner,pixset,4);
       I nv1=pixset.nval();
       nbase.query_polygon_inclusive(corner,pixset,4);
       I nv2=pixset.nval();
       if (nv1<nv2)
-        cout << "  PROBLEM: inclusive(RING)<inclusive(NEST): "
-             << nv1 << " vs. " << nv2 << endl;
+        FAIL(cout << "  PROBLEM: inclusive(RING)<inclusive(NEST): "
+                  << nv1 << " vs. " << nv2 << endl)
       if (nv2<nval)
-        cout << "  PROBLEM: inclusive(NEST)<non-inclusive: "
-             << nv2 << " vs. " << nval << endl;
+        FAIL(cout << "  PROBLEM: inclusive(NEST)<non-inclusive: "
+                  << nv2 << " vs. " << nval << endl)
       }
     }
   }
@@ -408,7 +797,7 @@ void helper_oop (int order)
   map3.Import(map2);
   for (int m=0; m<map.Npix(); ++m)
     if (!approx(map[m],map3[m],1e-12))
-      cout << "PROBLEM: order = " << order << endl;
+      FAIL(cout << "PROBLEM: order = " << order << endl)
   }
 void helper_udgrade (int order, Healpix_Ordering_Scheme s1,
   Healpix_Ordering_Scheme s2)
@@ -419,7 +808,7 @@ void helper_udgrade (int order, Healpix_Ordering_Scheme s1,
   map3.Import(map2);
   for (int m=0; m<map.Npix(); ++m)
     if (!approx(map[m],map3[m],1e-12))
-      cout << "PROBLEM: order = " << order << endl;
+      FAIL(cout << "PROBLEM: order = " << order << endl)
   }
 void helper_udgrade2 (int nside)
   {
@@ -430,7 +819,7 @@ void helper_udgrade2 (int nside)
   map3.Import(map2);
   for (int m=0; m<map.Npix(); ++m)
     if (!approx(map[m],map3[m],1e-12))
-      cout << "PROBLEM: nside = " << nside << endl;
+      FAIL(cout << "PROBLEM: nside = " << nside << endl)
   }
 
 void check_import()
@@ -460,8 +849,8 @@ void check_average()
       map[m] = rng.rand_uni()+0.01;
     map2.Import(map);
     double avg=map.average(), avg2=map2.average();
-    if (!approx(avg,avg2,1e-14))
-      cout << "PROBLEM: order = " << order << " " << avg/avg2-1 << endl;
+    if (!approx(avg,avg2,1e-13))
+      FAIL(cout << "PROBLEM: order = " << order << " " << avg/avg2-1 << endl)
     }
   for (int nside=3; nside<1000; nside += nside/2+1)
     {
@@ -470,8 +859,8 @@ void check_average()
       map[m] = rng.rand_uni()+0.01;
     map2.Import(map);
     double avg=map.average(), avg2=map2.average();
-    if (!approx(avg,avg2,1e-14))
-      cout << "PROBLEM: nside = " << nside << " " << avg/avg2-1 << endl;
+    if (!approx(avg,avg2,1e-13))
+      FAIL(cout << "PROBLEM: nside = " << nside << " " << avg/avg2-1 << endl)
     }
   }
 
@@ -482,17 +871,17 @@ void random_alm (Alm<xcomplex<double> >&almT, Alm<xcomplex<double> >&almG,
 
   for (int l=0; l<=lmax; ++l)
     {
-    almT(l,0).re=rng.rand_gauss(); almT(l,0).im=0.;
-    almG(l,0).re=rng.rand_gauss(); almG(l,0).im=0.;
-    almC(l,0).re=rng.rand_gauss(); almC(l,0).im=0.;
+    almT(l,0)=dcomplex(rng.rand_gauss(),0.);
+    almG(l,0)=dcomplex(rng.rand_gauss(),0.);
+    almC(l,0)=dcomplex(rng.rand_gauss(),0.);
     }
 
   for (int m=1; m<=mmax; ++m)
     for (int l=m; l<=lmax; ++l)
       {
-      almT(l,m).re=rng.rand_gauss(); almT(l,m).im=rng.rand_gauss();
-      almG(l,m).re=rng.rand_gauss(); almG(l,m).im=rng.rand_gauss();
-      almC(l,m).re=rng.rand_gauss(); almC(l,m).im=rng.rand_gauss();
+      almT(l,m).real(rng.rand_gauss()); almT(l,m).imag(rng.rand_gauss());
+      almG(l,m).real(rng.rand_gauss()); almG(l,m).imag(rng.rand_gauss());
+      almC(l,m).real(rng.rand_gauss()); almC(l,m).imag(rng.rand_gauss());
       }
   almG(0,0)=almC(0,0)=almG(1,0)=almC(1,0)=almG(1,1)=almC(1,1)=0;
   }
@@ -502,11 +891,11 @@ void random_alm (Alm<xcomplex<double> >&alm, int lmax, int mmax)
   alm.Set(lmax,mmax);
 
   for (int l=0; l<=lmax; ++l)
-    { alm(l,0).re=rng.rand_gauss(); alm(l,0).im=0.; }
+    { alm(l,0)=dcomplex(rng.rand_gauss(),0.); }
 
   for (int m=1; m<=mmax; ++m)
     for (int l=m; l<=lmax; ++l)
-      { alm(l,m).re=rng.rand_gauss(); alm(l,m).im=rng.rand_gauss(); }
+      { alm(l,m).real(rng.rand_gauss()); alm(l,m).imag(rng.rand_gauss()); }
   }
 
 void check_alm (const Alm<xcomplex<double> >&oalm,
@@ -515,10 +904,10 @@ void check_alm (const Alm<xcomplex<double> >&oalm,
   for (int m=0; m<=alm.Mmax(); ++m)
     for (int l=m; l<=alm.Lmax(); ++l)
       {
-      if (!abs_approx(oalm(l,m).re,alm(l,m).re,epsilon))
-        cout << "Problemr " << l << " " << m << endl;
-      if (!abs_approx(oalm(l,m).im,alm(l,m).im,epsilon))
-        cout << "Problemi " << l << " " << m <<  endl;
+      if (!abs_approx(oalm(l,m).real(),alm(l,m).real(),epsilon))
+        FAIL(cout << "Problemr " << l << " " << m << endl)
+      if (!abs_approx(oalm(l,m).imag(),alm(l,m).imag(),epsilon))
+        FAIL(cout << "Problemi " << l << " " << m <<  endl)
       }
   }
 
@@ -591,8 +980,8 @@ void check_isqrt()
   {
   cout << "testing whether isqrt() works reliably" << endl;
   uint64 val=uint64(0xF234)<<16, valsq=val*val;
-  if (isqrt(valsq)!=val) cout << "PROBLEM1" << endl;
-  if (isqrt(valsq-1)!=val-1) cout << "PROBLEM2" << endl;
+  if (isqrt(valsq)!=val) FAIL(cout << "PROBLEM1" << endl)
+  if (isqrt(valsq-1)!=val-1) FAIL(cout << "PROBLEM2" << endl)
   }
 
 void check_pix2ang_acc()
@@ -602,7 +991,7 @@ void check_pix2ang_acc()
     {
     Healpix_Base2 base(m,RING);
     if (base.pix2ang(1).theta==0.)
-      cout << "PROBLEM: order " << m << endl;
+      FAIL(cout << "PROBLEM: order " << m << endl)
     }
   }
 
@@ -813,7 +1202,7 @@ template<typename I>void perf_query_disc(const string &name,
     {
     rangeset<I> pix;
     base.query_disc(vec3(1,0,0),halfpi/9,pix);
-    dummy+=pix.size();
+    dummy+=pix.nranges();
     ++cnt;
     }
   wallTimers.stop(name);
@@ -833,7 +1222,7 @@ template<typename I>void perf_query_triangle(const string &name,
     {
     rangeset<I> pix;
     base.query_polygon(corner,pix);
-    dummy+=pix.size();
+    dummy+=pix.nranges();
     ++cnt;
     }
   wallTimers.stop(name);
@@ -854,7 +1243,7 @@ template<typename I>void perf_query_polygon(const string &name,
     {
     rangeset<I> pix;
     base.query_polygon(corner,pix);
-    dummy+=pix.size();
+    dummy+=pix.nranges();
     ++cnt;
     }
   wallTimers.stop(name);
@@ -912,12 +1301,21 @@ void perftest()
 
   if (dummy<0) cout << dummy << endl;
   }
+
 } // unnamed namespace
 
 int main(int argc, const char **argv)
   {
   module_startup ("hpxtest",argc,argv,1,"");
   perftest();
+  check_compress<int>();
+  check_compress<unsigned>();
+  check_compress<int64>();
+  check_compress<uint64>();
+  check_Moc<int>();
+  check_Moc<int64>();
+  check_rangeset<int>();
+  check_rangeset<int64>();
   check_isqrt();
   check_pix2ang_acc();
   check_smooth_alm();
@@ -941,8 +1339,13 @@ int main(int argc, const char **argv)
   check_swap_scheme();
   check_query_disc_strict(RING);
   check_query_disc_strict(NEST);
+  check_issue_229(RING);
+  check_issue_229(NEST);
   check_query_disc<int>();
   check_query_disc<int64>();
   check_query_polygon<int>();
   check_query_polygon<int64>();
+#ifdef UNITTESTS
+  if (errcount>0) planck_fail("unit test errors");
+#endif
   }
